@@ -1,25 +1,21 @@
-package Test::Chimps;
-
-use 5.008;
-
 =head1 NAME
 
 Test::Chimps - Collaborative Heterogeneous Infinite Monkey Perfectionification Service
 
 =head1 VERSION
 
-Version 0.07_01
+Version 0.02
 
 =cut
 
-our $VERSION = '0.07_01';
+our $VERSION = '0.02';
 
 =head1 SYNOPSIS
 
 Why run tests yourself?  Let an infinite number of monkeys do it
-for you!  Take the monkey work out of testing!  Remove the monkey
-wrench from your development process!  Alright, I'm done.  Sorry
-about that.  I got a little carried away...
+for you!  Take the monkey work out of testing.  Remove the monkey
+wrench from your development process.  Will the monkey jokes ever
+end?  No!  We really are more fun than a barrel full of monkeys!
 
 The Collaborative Heterogeneous Infinite Monkey Perfectionification
 Service (CHIMPS) is a generalized testing framework designed to
@@ -84,13 +80,6 @@ You can also look for information at:
 
 =over 4
 
-=item * Mailing list
-
-Chimps has a mailman mailing list at
-L<chimps@bestpractical.com>.  You can subscribe via the web
-interface at
-L<http://lists.bestpractical.com/cgi-bin/mailman/listinfo/chimps>.
-
 =item * AnnoCPAN: Annotated CPAN documentation
 
 L<http://annocpan.org/dist/Test-Chimps>
