@@ -9,7 +9,7 @@ Test::Chimps::Report - Encapsulate a smoke test report
 
 =head1 SYNOPSIS
 
-Represents a smoke report.  See L<Jifty::DBI::Record>.
+FIXME
 
 Note that parts of this class are dynamically generated in
 Test::Chimps::Server base on the configuation file.
@@ -56,29 +56,31 @@ C<Test::Chimps::Report>s have the following columns (and consequently accessors)
 =back
 
 Additionally, columns are added dynamically based on the report
-variables specified in the server.  Unfortunately, this means that
-external modules have a hard time getting at the C<Report> class as
-seen by the server.
+variables specified in the server.
 
 =cut
 
 use base qw/Jifty::DBI::Record/;
 
+package Test::Chimps::Report::Schema;
+
 use Jifty::DBI::Schema;
-use Jifty::DBI::Record schema {
 
 column report_html                  => type is 'text';
+column model_structure              => type is 'text',
+  filters are 'Jifty::DBI::Filter::Storable', 'Jifty::DBI::Filter::base64';
 column timestamp                    => type is 'timestamp',
   filters are 'Jifty::DBI::Filter::DateTime';
+column total_ok                     => type is 'integer';
 column total_passed                 => type is 'integer';
+column total_nok                    => type is 'integer';
 column total_failed                 => type is 'integer';
-column total_ratio                  => type is 'float';
+column total_percentage             => type is 'integer';
+column total_ratio                  => type is 'integer';
 column total_seen                   => type is 'integer';
 column total_skipped                => type is 'integer';
 column total_todo                   => type is 'integer';
 column total_unexpectedly_succeeded => type is 'integer';
-column duration                     => type is 'integer';
-};
 
 =head1 AUTHOR
 
@@ -102,13 +104,6 @@ You can also look for information at:
 
 =over 4
 
-=item * Mailing list
-
-Chimps has a mailman mailing list at
-L<chimps@bestpractical.com>.  You can subscribe via the web
-interface at
-L<http://lists.bestpractical.com/cgi-bin/mailman/listinfo/chimps>.
-
 =item * AnnoCPAN: Annotated CPAN documentation
 
 L<http://annocpan.org/dist/Test-Chimps>
